/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

// for Prototype 1.0 circuit
//#define PROTOTYPE_1

#ifndef USBJOYSTICK_C
#define USBJOYSTICK_C

/** INCLUDES *******************************************************/
#include <usb/usb.h>
#include <usb/usb_device_hid.h>

#include <system.h>

//#include <app_led_usb_status.h>

#include <stdint.h>

#include "joystick.h"

/** DECLARATIONS ***************************************************/
//http://www.microsoft.com/whdc/archive/hidgame.mspx
#define HAT_SWITCH_NORTH            0x0
#define HAT_SWITCH_NORTH_EAST       0x1
#define HAT_SWITCH_EAST             0x2
#define HAT_SWITCH_SOUTH_EAST       0x3
#define HAT_SWITCH_SOUTH            0x4
#define HAT_SWITCH_SOUTH_WEST       0x5
#define HAT_SWITCH_WEST             0x6
#define HAT_SWITCH_NORTH_WEST       0x7
#define HAT_SWITCH_NULL             0x8


/** VARIABLES ******************************************************/
/* Some processors have a limited range of RAM addresses where the USB module
 * is able to access.  The following section is for those devices.  This section
 * assigns the buffers that need to be used by the USB module into those
 * specific areas.
 */
#if defined(FIXED_ADDRESS_MEMORY)
    #if defined(COMPILER_MPLAB_C18)
        #pragma udata JOYSTICK_DATA=JOYSTICK_DATA_ADDRESS
            INPUT_CONTROLS joystick_input;
        #pragma udata
    #elif defined(__XC8)
        INPUT_CONTROLS joystick_input @ JOYSTICK_DATA_ADDRESS;
    #endif
#else
    INPUT_CONTROLS joystick_input;
#endif

extern bool RAPID_FIRE;

USB_VOLATILE USB_HANDLE lastTransmission = 0;

void Set_AnalogAndHat(char right, char left, char up, char down);

/*********************************************************************
* Function: void APP_DeviceJoystickInitialize(void);
*
* Overview: Initializes the demo code
*
* PreCondition: None
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceJoystickInitialize(void)
{
    //initialize the variable holding the handle for the last
    // transmission
    lastTransmission = 0;

    //enable the HID endpoint
    USBEnableEndpoint(JOYSTICK_EP,USB_IN_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
}//end UserInit

/*********************************************************************
* Function: void APP_DeviceJoystickTasks(void);
*
* Overview: Keeps the demo running.
*
* PreCondition: The demo should have been initialized and started via
*   the APP_DeviceJoystickInitialize() and APP_DeviceJoystickStart() demos
*   respectively.
*
* Input: None
*
* Output: None
*
********************************************************************/






void APP_DeviceJoystickTasks(void)
{
    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;

    //If the last transmision is complete
    if(!HIDTxHandleBusy(lastTransmission))
    {
        //Send the packet over USB to the host.
        lastTransmission = HIDTxPacket(JOYSTICK_EP, (uint8_t*)&joystick_input, sizeof(joystick_input));
    }

}//end ProcessIO

#endif

//
//
//  joystick_inputを更新する
//
//
void RenewJoystickState()
{
    static char devType = 0;

    int buttons;
    static int last_buttons  = 0;
    static int rapid_setting = 0;
    
    if(devType == DEVTYPE_UNKNOWN)
    {
        ReadFightingPad6B(&devType, &buttons);

#ifndef Enable_PS
        if(devType == DEVTYPE_UNKNOWN)
        {
            ReadSS(&devType, &buttons);
        }
#endif

        if(devType == DEVTYPE_UNKNOWN)
        {
            ReadFamicomJoycard(&devType, &buttons);
        }

#ifndef Enable_PS
        if(devType == DEVTYPE_UNKNOWN)
        {
            ReadKabutogani(&devType, &buttons);
        }
#endif

#ifdef Enable_PS
        if(devType == DEVTYPE_UNKNOWN)
        {
            ReadPS(&devType, &buttons);
        }
#endif

    }
    else
    {
        // １度デバイスを特定できたら、２回目以降は即そのデバイスで読み
        switch(devType)
        {
            case DEVTYPE_MD:
                ReadFightingPad6B(&devType, &buttons);
                break;

#ifndef Enable_PS
            case DEVTYPE_SS:
                ReadSS(&devType, &buttons);
                break;
#endif

            case DEVTYPE_FAMICOM:
                ReadFamicomJoycard(&devType, &buttons);
                break;

#ifndef Enable_PS
            case DEVTYPE_KABUTOGANI:
                ReadKabutogani(&devType, &buttons);
                break;
#endif

#ifdef Enable_PS
            case DEVTYPE_PS:
                ReadPS(&devType, &buttons);
                break;
#endif
        }
    }

    // ここまででボタンが読み取れているはず

    // 連射設定更新
    if( !(IN_RAPPIDSET))
    {
        // 本体のラピッドボタンが押されている

        for(int i = 0; i < 16; i++)
        {
            // 0->1に変わったビットを探す
            if(
                (((last_buttons>>i) & 0x0001) == 0) &&
                (((buttons>>i) & 0x0001) == 1)
            )
            {
                int newBit = (~((rapid_setting>>i) & 0x0001)) & 0x0001;
                rapid_setting = (rapid_setting & ~((int)(0x0001<<i))) | (newBit<<i);
            }
        }
    }

    // 連射用ボタンデータを作成
    int writeVal = 0;
    for(int i = 0; i < 16; i++)
    {
        if( ((buttons>>i) & 0x01) )
        {
            if( ((rapid_setting>>i) & 0x01) )
            {
                writeVal = (writeVal & ~((int)(0x0001<<i))) | ((RAPID_FIRE)? (int)(0x0001<<i) : 0);
            }
            else
            {
                writeVal = (writeVal & ~((int)(0x0001<<i))) | (int)(0x0001<<i);
            }
        }
    }

    // 出力用領域に書き込み
    joystick_input.val[0] = (char)(writeVal & 0x00ff);
    joystick_input.val[1] = (char)((writeVal & 0xff00)>>8);

    last_buttons = buttons;
}

/*
 *
 * 共用ルーチン
 *
 */


unsigned char Get_HatSwitchVal(char right, char left, char up, char down)
{
    //Hat switch
    unsigned char hatswitch = 0x0f;

    if (!up)    // up
    {
        hatswitch = 0;
        if (!right) // right
        {
            hatswitch = 1;
        }

        if (!left)  // left
        {
            hatswitch = 7;
        }
    }
    else if (!right)    // right
    {
        hatswitch = 2;
        if (!down)   // down
        {
            hatswitch = 3;
        }
    }
    else if (!down) // down
    {
        hatswitch = 4;
        if (!left)  // left
        {
            hatswitch = 5;
        }
    }
    else if (!left) // left
    {
        hatswitch = 6;
    }
    else
    {
        hatswitch = 0xf;
    }

    return hatswitch;
}


void Set_AnalogAndHatValue
    (char right, char left, char up, char down)
{
    //Hat switch newtral
    unsigned char hatswitch = 0x0f;
    unsigned char val3 = 0x7f;
    unsigned char val4 = 0x7f;
    unsigned char val5 = 0x7f;
    unsigned char val6 = 0x7f;

    if(IN_SEL_ANALOG_HAT)
    {
        // Output as Analog stick
        if (!right) // Right
        {
            val3 = 0xff;
        }
        else if (!left) // Left
        {
            val3 = 0x00;
        }

        if (!up)    // Up
        {
            val4 = 0x00;
        }
        else if (!down) // Down
        {
            val4 = 0xff;
        }

    }
    else
    {
        hatswitch = Get_HatSwitchVal(right, left, up, down);
    }

    joystick_input.val[2] = hatswitch;
    joystick_input.val[3] = val3;
    joystick_input.val[4] = val4;
    joystick_input.val[5] = val5;
    joystick_input.val[6] = val6;
}
