/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

/** INCLUDES *******************************************************/
#include <system.h>

#include <usb/usb.h>
#include <usb/usb_device_hid.h>

#include <app_device_joystick.h>
//#include <app_led_usb_status.h>


bool G_USBINT = false;
bool RAPID_FIRE = true;


// FightingPad6B対応
// ファミコンコントローラ対応
// サターンパッド＆マルコン(デジタル/アナログモード)対応
// サイバースティック(アナログ/デジタルモード)対応
MAIN_RETURN main(void)
{
    SYSTEM_Initialize(SYSTEM_STATE_USB_START);

    USBDeviceInit();
    USBDeviceAttach();

    ANSEL  = 0;                 // アナログ設定レジスタ　 すべてデジタル
    ANSELH = 0;                 // アナログ設定レジスタH　すべてデジタル

    // PortB設定
    TRISB = 0b11110000;         // 上位4ピン=入力
/*
    // Timer
    T1CON  = 0b10000001;
    //T1CON  = 0b10100001;
    TMR1H = 0x80;
    TMR1L = 0x00;
    IPR1bits.TMR1IP = 0;    // Timer 1 からの割込みを低優先に設定
    //IPR1bits.TMR1IP = 1;    // Timer 1 からの割込みを高優先に設定
    PIR1bits.TMR1IF = 0;    // Timer 1 からの割込みフラッグをクリア
    PIE1bits.TMR1IE = 1;    // Timer 1 からの割込みを許可

    RCONbits.IPEN=1;        // ２段階の割込みに設定
    INTCONbits.GIEH=1;      // 高優先割込みを許可
    INTCONbits.GIEL=1;      // 低優先割込みを許可
*/
    while(1)
    {
        SYSTEM_Tasks();

        #if defined(USB_POLLING)
            // Interrupt or polling method.  If using polling, must call
            // this function periodically.  This function will take care
            // of processing and responding to SETUP transactions
            // (such as during the enumeration process when you first
            // plug in).  USB hosts require that USB devices should accept
            // and process SETUP packets in a timely fashion.  Therefore,
            // when using polling, this function should be called
            // regularly (such as once every 1.8ms or faster** [see
            // inline code comments in usb_device.c for explanation when
            // "or faster" applies])  In most cases, the USBDeviceTasks()
            // function does not take very long to execute (ex: <100
            // instruction cycles) before it returns.
            USBDeviceTasks();
        #endif

        /* If the USB device isn't configured yet, we can't really do anything
         * else since we don't have a host to talk to.  So jump back to the
         * top of the while loop. */
        if( USBGetDeviceState() < CONFIGURED_STATE )
        {
            /* Jump back to the top of the while loop. */
            continue;
        }

        /* If we are currently suspended, then we need to see if we need to
         * issue a remote wakeup.  In either case, we shouldn't process any
         * keyboard commands since we aren't currently communicating to the host
         * thus just continue back to the start of the while loop. */
        if( USBIsDeviceSuspended() == true )
        {
            /* Jump back to the top of the while loop. */
            continue;
        }

        //Application specific tasks
        APP_DeviceJoystickTasks();


        RenewJoystickState();

        RAPID_FIRE = !RAPID_FIRE;           // invert value
        
    }//end while
}//end main


bool USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, uint16_t size)
{
    switch( (int) event )
    {
        case EVENT_TRANSFER:
            break;

        case EVENT_SOF:
            /* We are using the SOF as a timer to time the LED indicator.  Call
             * the LED update function here. */
        //    APP_LEDUpdateUSBStatus();
            break;

        case EVENT_SUSPEND:
            /* Update the LED status for the suspend event. */
        //    APP_LEDUpdateUSBStatus();
            break;

        case EVENT_RESUME:
            /* Update the LED status for the resume event. */
        //    APP_LEDUpdateUSBStatus();
            break;

        case EVENT_CONFIGURED:
            /* When the device is configured, we can (re)initialize the demo
             * code. */
            APP_DeviceJoystickInitialize();
            break;

        case EVENT_SET_DESCRIPTOR:
            break;

        case EVENT_EP0_REQUEST:
            /* We have received a non-standard USB request.  The HID driver
             * needs to check to see if the request was for it. */
            USBCheckHIDRequest();
            break;

        case EVENT_BUS_ERROR:
            break;

        case EVENT_TRANSFER_TERMINATED:
            break;

        default:
            break;
    }

    G_USBINT = true;

    return true;
}
/*
//
// 低優先度割り込みISR
//
void interrupt low_priority Lo_Isr(void)
{
    
    // Timer ISR
    if(PIR1bits.TMR1IF)
    {
        // Interrupt from Timer 1

        // Set Timer1 initial counter value
        TMR1H = 0x80;
        TMR1L = 0x00;
        PIR1bits.TMR1IF = 0;        // Clear interrupt flag

        RAPID_FIRE = !RAPID_FIRE;           // invert value
    }
    
}
*/
/*******************************************************************************
 End of File
*/

