// 現状のバージョンでは一部、Digitalモードで不具合があるが
// このままとする。
//
// Digitalモードで「スロットル引く」を行うと、
// ボタン3,4,5,6,13が押された状態になり、かつ十字キー右上になってしまう。
// 他ジョイスティック読み取りルーチンが誤判定しているかも？

#include "joystick.h"

#define LOOPMAX 1200    // 50で8.7us    162.56us待つ時もあり
#define WAIT_CS    Delay10TCYx(25)   // 27.7us
#define WAIT_CS_D  Delay10TCYx(6)   // 10us
#define CS_DATA    ((PORTC) & 0x0f)
#define CS_LH      (PORTCbits.RC4)
#define CS_ACK     (PORTCbits.RC6)
#define CS_REQ_L     LATC = 0x00
#define CS_REQ_H     LATC = 0x20
#define CS_WAIT_NEXTPOLL    Delay1KTCYx(61) // 83usec x 61 = 5063usec = 5msec
//#define CS_WAIT_NEXTPOLL    Delay1KTCYx(90) // 83usec x 90 = 7470usec = 7.4msec

// Prottype　definition
void PrepareForReadKabutogani(void);
bool ReadPhaseDataKG(unsigned char *phaseData);


/// -------------------------------------------------------
/// 読み取り準備（ポート設定など）
/// -------------------------------------------------------
void PrepareForReadKabutogani(void)
{
    // PortC Initialize port IN/OUT direction
    TRISC = 0b11011111;         // SEL出力、他は入力

    // 準備
    CS_REQ_H;
    WAIT_PORTSET;
}


/// -------------------------------------------------------
/// 読み取り＆ジョイスティック出力データ作成
/// （サイバースティック/カブトガニ）
/// -------------------------------------------------------
bool ReadKabutogani
    (char *devType, int *buttons)
{
	// 読み取りを行うと、以下の3ケースいずれか
	// 1.アナログ読み成功かつUSB割り込み無し
	// 2.アナログ読み成功かつUSB割り込み有り
	// 3.アナログ読み失敗(長いので、USB割り込み常に有り？)
	// 
	// デバイス判定
	//   初期状態：不明なデバイスとする。
	//
	//   ①読み取り処理
	//     結果により分岐
	//     1の場合→アナログスティックと判定。エラーカウント0リセット
	//              ②-aへ
	//
	//     2,3の場合→エラーカウント20未満だったら、
	//                  ＋1し、何もしないでreturn
	//                さもなくば、
	//                  ②-bへ
	//
	//   ②-a アナログスティックの情報をUSB出力
	//   ②-b デジタルスティックの情報をUSB出力
    unsigned char phaseData[12];
    unsigned char phaseData_D[2];
    bool isAnalog = false;
    bool state = true;

    static unsigned int counter_Failed = 0;

	// 不明デバイスだったら、初期化処理
    if(*devType == DEVTYPE_UNKNOWN)
    {
        PrepareForReadKabutogani();
    }

	// USB割り込み発生フラグ
	//		もしOFF後、勝手にONに変わってたら
	//		USB読み取りが発生したことを示す
    G_USBINT = false;	// USB割り込み発生フラグOFF

	// ①読み取り処理
    // Read
    state = ReadPhaseDataKG(phaseData);


	// 結果により分岐
    if(state && G_USBINT == false)
    {
		// 1.アナログ読み成功かつUSB割り込み無し
		counter_Failed = 0;
        isAnalog = true;
        *devType = DEVTYPE_KABUTOGANI;
	}
	else
	{
		// 2.アナログ読み成功かつUSB割り込み有り
		// 3.アナログ読み失敗(長いので、USB割り込み常に有り？)
		if(counter_Failed < 20)
		{
			counter_Failed++;
			return;
		}
		else
		{
			isAnalog = false;
	        *devType = DEVTYPE_UNKNOWN;
		}
    }
	
    // USB出力
    if(isAnalog)
    {

        //Buttons
        *buttons = 0x00000000 |
            ((!(phaseData[0]&0b0010) &&  (phaseData[0]&0b0001)) ? BTNBIT_01 : 0) | // C
            ((!(phaseData[0]&0b0100) &&  (phaseData[0]&0b0001)) ? BTNBIT_02 : 0) |  // B
            ((!(phaseData[0]&0b1000) &&  (phaseData[0]&0b0001)) ? BTNBIT_03 : 0) |  // A
            ((!(phaseData[0]&0b0100) && !(phaseData[0]&0b0001)) ? BTNBIT_04 : 0) |  // B+D
            ((!(phaseData[1]&0b1000) &&  (phaseData[0]&0b0001)) ? BTNBIT_05 : 0) |  // E1
            ((!(phaseData[1]&0b0100) &&  (phaseData[0]&0b0001)) ? BTNBIT_06 : 0) |  // E2
            ((!(phaseData[1]&0b1000) && !(phaseData[0]&0b0001)) ? BTNBIT_07 : 0) |  // E1+D
            ((!(phaseData[1]&0b0100) && !(phaseData[0]&0b0001)) ? BTNBIT_08 : 0) |  // E2+D
            ((!(phaseData[1]&0b0001) &&  (phaseData[0]&0b0001)) ? BTNBIT_09 : 0) |  // G(Mode)
            ((!(phaseData[1]&0b0010) &&  (phaseData[0]&0b0001)) ? BTNBIT_10 : 0) |  // F(Start)
            ((!(phaseData[0]&0b1000) && !(phaseData[0]&0b0001)) ? BTNBIT_11 : 0) |  // A+D
            ((!(phaseData[0]&0b0010) && !(phaseData[0]&0b0001)) ? BTNBIT_12 : 0) |  // C+D
            ((!(phaseData[1]&0b0010) && !(phaseData[0]&0b0001)) ? BTNBIT_13 : 0) ;  // F(Start)+D

        //Analog sticks
        joystick_input.val[3] = (phaseData[ 3]<<4) | phaseData[ 7]; // X
        joystick_input.val[4] = (phaseData[ 2]<<4) | phaseData[ 6]; // Y
        joystick_input.val[5] = (phaseData[ 5]<<4) | phaseData[ 9]; // Z
        joystick_input.val[6] = (phaseData[ 4]<<4) | phaseData[ 8]; // Rz

        //Hat switch
        unsigned char hatswitch = 0x0f;
        bool blnUp      = ((joystick_input.val[4]<0x10) && !(phaseData[0]&0b0001));
        bool blnDown    = ((joystick_input.val[4]>0xf0) && !(phaseData[0]&0b0001));
        bool blnRight   = ((joystick_input.val[3]>0xf0) && !(phaseData[0]&0b0001));
        bool blnLeft    = ((joystick_input.val[3]<0x10) && !(phaseData[0]&0b0001));

        // If D pressed, set analog stick to center. And set Rz
        if(!(phaseData[0]&0b0001))
        {
            joystick_input.val[3] = 0x7f;
            joystick_input.val[4] = 0x7f;
            joystick_input.val[5] = (phaseData[ 4]<<4) | phaseData[ 8];
            joystick_input.val[6] = 0x7f;
        }

        if ( blnUp )    // up
        {
            hatswitch = 0;
            if ( blnRight ) // right
            {
                hatswitch = 1;
            }
            if ( blnLeft)  // left
            {
                hatswitch = 7;
            }
        }
        else if ( blnRight )    // right
        {
            hatswitch = 2;
            if ( blnDown )   // down
            {
                hatswitch = 3;
            }
        }
        else if ( blnDown ) // down
        {
            hatswitch = 4;
            if ( blnLeft )  // left
            {
                hatswitch = 5;
            }
        }
        else if ( blnLeft ) // left
        {
            hatswitch = 6;
        }
        else
        {
            hatswitch = 0xf;
        }
        joystick_input.val[2] = hatswitch;

        //Send the packet over USB to the host.
        //lastTransmission = HIDTxPacket(JOYSTICK_EP, (uint8_t*)&joystick_input, sizeof(joystick_input));
    }
    else
    {
        phaseData_D[1] = (PORTC & 0x0f) | (CS_LH << 4) | (CS_ACK << 5);
        CS_REQ_L;
        WAIT_CS_D;
        phaseData_D[0] = (PORTC & 0x0f) | (CS_LH << 4) | (CS_ACK << 5);
        CS_REQ_H;
		
        //Buttons
        *buttons = 0x00000000 |
            ((phaseData_D[0]&0b010000) ? 0 : BTNBIT_01) |  // A
            ((phaseData_D[0]&0b100000) ? 0 : BTNBIT_02) |  // B
            ((phaseData_D[1]&0b000001) ? 0 : BTNBIT_03) |  // ThrUp
            ((phaseData_D[1]&0b000010) ? 0 : BTNBIT_04) |  // ThrDown
            ((phaseData_D[1]&0b000100) ? 0 : BTNBIT_05) |  // C
            ((phaseData_D[1]&0b001000) ? 0 : BTNBIT_06) |  // D
            ((phaseData_D[1]&0b010000) ? 0 : BTNBIT_07) |  // E1
            ((phaseData_D[1]&0b100000) ? 0 : BTNBIT_08) ;  // E2

        Set_AnalogAndHatValue(
                (phaseData_D[0]&0b001000),     // R
                (phaseData_D[0]&0b000100),     // L
                (phaseData_D[0]&0b000001),     // U
                (phaseData_D[0]&0b000010));    // D
    }
	
	// 次回読み取りまで5msecかかるようにする
    CS_WAIT_NEXTPOLL;
	
    return true;
}

/// --------------------------------------------------------------------------
/// Read function
/// --------------------------------------------------------------------------
bool ReadPhaseDataKG(unsigned char *phaseData)
{
    unsigned int  count = 0;
    bool state = true;

	/*
	// 1/4倍速モード
	// 下げっぱなしにする
    CS_REQ_L;
	*/
	
	
    // REQ 立ち下げ＆上げ
    CS_REQ_L;
    WAIT_CS;
    CS_REQ_H;
	

    for(int i=0; i<12; i+=2)
    {
        // wait ACK==L
        count = 0;
        while(CS_ACK == 1)
        {
            count++;
            if(count > LOOPMAX)
            {
                state = false;
                break;
            }
        }

        if(!state)
        {
            break;
        }

        // ACK==L縺ｨ縺ｪ縺｣縺�
        phaseData[i] = CS_DATA;

        // WAIT LH==1
        count = 0;
        while(CS_LH == 0)
        {
            count++;
            if(count > LOOPMAX)
            {
                state = false;
                break;
            }
        }

        if(!state)
        {
            break;
        }

        // wait ACK==L
        count = 0;
        while(CS_ACK == 1)
        {
            count++;
            if(count > LOOPMAX)
            {
                state = false;
                break;
            }
        }

        if(!state)
        {
            break;
        }

        // ACK==L縺ｨ縺ｪ縺｣縺�
        phaseData[i+1] = CS_DATA;

        // wait LH==L
        count = 0;
        while(CS_LH == 1)
        {
            count++;
            if(count > LOOPMAX)
            {
                state = false;
                break;
            }
        }

        if(!state)
        {
            break;
        }
    }
	
	/*
	// 1/4倍速モード
	// 立ち上げる
    CS_REQ_H;
	*/
	
    return state;
}
